from typing import Any, Dict, Union

LOCATIONS_URL: Dict[str, Dict[Union[float, int], str]] = {
    "AC": {
        68.1: "rio-branco",
        68.2: "outras-cidades",
    },
    "AL": {82.1: "maceio", 82.2: "outras-cidades"},
    "AP": {
        96.1: "macapa",
        96.2: "outras-cidades",
    },
    "AM": {
        92: "regiao-de-manaus",
        97: "leste-do-amazonas",
    },
    "BA": {
        71: "grande-salvador",
        73: "sul-da-bahia",
        74: "regiao-de-juazeiro-e-jacobina",
        75: "regiao-de-feira-de-santana-e-alagoinhas",
        77: "regiao-de-vitoria-da-conquista-e-barreiras",
    },
    "CE": {
        85: "fortaleza-e-regiao",
        88: "regiao-de-juazeiro-do-norte-e-sobral",
    },
    "DF": {
        61.1: "Brasilia",
        61.2: "outras-cidades",
    },
    "ES": {
        27: "norte-do-espirito-santo",
        28: "sul-do-espirito-santo",
    },
    "GO": {
        62: "grande-goiania-e-anapolis",
        64: "regiao-de-rio-verde-e-caldas-novas",
    },
    "MA": {
        98: "regiao-de-sao-luis",
        99: "regiao-de-imperatriz-e-caxias",
    },
    "MT": {
        65: "regiao-de-cuiaba",
        66: "regiao-de-rondonopolis-e-sinop",
    },
    "MS": {
        67.1: "campo-grande",
        67.2: "corumba",
        67.3: "ourados",
        67.4: "outras-cidades",
        67.5: "tres-lagoas",
    },
    "MG": {
        31: "belo-horizonte-e-regiao",
        32: "regiao-de-juiz-de-fora",
        33: "regiao-de-governador-valadares-e-teofilo-otoni",
        34: "regiao-de-uberlandia-e-uberaba",
        35: "regiao-de-pocos-de-caldas-e-varginha",
        37: "regiao-de-divinopolis",
        38: "regiao-de-montes-claros-e-diamantina",
    },
    "PA": {
        91: "regiao-de-belem",
        93: "regiao-de-santarem",
        94: "regiao-de-maraba",
    },
    "PB": {
        83.1: "joao-pessoa",
        83.2: "campina-grande-guarabira-e-regiao",
        83.3: "monteiro-picui-e-regiao",
        83.4: "patos-sousa-e-regiao",
        83.5: "santa-rita-bayeux-e-regiao",
    },
    "PR": {
        41: "regiao-de-curitiba-e-paranagua",
        42: "regiao-de-ponta-grossa-e-guarapuava",
        43: "regiao-de-londrina",
        44: "regiao-de-maringa",
        45: "regiao-de-foz-do-iguacu-e-cascavel",
        46: "regiao-de-francisco-beltrao-e-pato-branco",
    },
    "PE": {
        81: "grande-recife",
        87: "regiao-de-petrolina-e-garanhuns",
    },
    "PI": {
        86: "regiao-de-teresina-e-parnaiba",
        89: "regiao-de-picos-e-floriano",
    },
    "RJ": {
        21: "rio-de-janeiro-e-regiao",
        22: "norte-do-estado-do-rio",
        24: "serra-angra-dos-reis-e-regiao",
    },
    "RN": {
        84.1: "natal",
        84.2: "outras-cidades",
    },
    "RS": {
        51: "regioes-de-porto-alegre-torres-e-santa-cruz-do-sul",
        53: "regioes-de-pelotas-rio-grande-e-bage",
        54: "regioes-de-caxias-do-sul-e-passo-fundo",
        55: "regioes-de-santa-maria-uruguaiana-e-cruz-alta",
    },
    "RO": {
        69.1: "porto-velho",
        69.2: "outras-cidades",
    },
    "RR": {
        95.1: "boa-vista",
        95.2: "outras-cidades",
    },
    "SC": {
        47: "norte-de-santa-catarina",
        48: "florianopolis-e-regiao",
        49: "oeste-de-santa-catarina",
    },
    "SP": {
        11: "sao-paulo-e-regiao",
        12: "vale-do-paraiba-e-litoral-norte",
        13: "baixada-santista-e-litoral-sul",
        14: "regiao-de-bauru-e-marilia",
        15: "regiao-de-sorocaba",
        16: "regiao-de-ribeirao-preto",
        17: "regiao-de-sao-jose-do-rio-preto",
        18: "regiao-de-presidente-prudente",
        19: "grande-campinas",
    },
    "SE": {
        79.1: "aracaju",
        79.2: "outras-cidades",
    },
    "TO": {
        63.1: "palmas",
        63.2: "outras-cidades",
    },
}

CATEGORIES: Dict = {
    "cars": {
        "category": "autos-e-pecas",
        "subcategories": {
            "parts": "pecas-e-acessorios",
            "cars": "carros-vans-e-utilitarios",
            "motorcycles": "motos",
            "trucks": "caminhoes",
            "bus": "onibus",
            "boats_planes": "barcos-e-aeronaves",
        },
    },
}

MILEAGE: Dict[str, int] = {
    "0": 0,
    "5000": 5_000,
    "10000": 10_000,
    "20000": 20_000,
    "30000": 30_000,
    "40000": 40_000,
    "60000": 60_000,
    "80000": 80_000,
    "100000": 100_000,
    "120000": 120_000,
    "140000": 140_000,
    "160000": 160_000,
    "180000": 180_000,
    "200000": 200_000,
    "250000": 250_000,
    "300000": 300_000,
    "400000": 400_000,
    "500000": 500_000,
}

YEAR: Dict[int, int] = {
    2022: 40,
    2021: 39,
    2020: 38,
    2019: 37,
    2018: 36,
    2017: 35,
    2016: 34,
    2015: 33,
    2014: 32,
    2013: 31,
    2012: 30,
    2011: 29,
    2010: 28,
    2009: 27,
    2008: 26,
    2007: 25,
    2006: 24,
    2005: 23,
    2004: 22,
    2003: 21,
    2002: 20,
    2001: 19,
    2000: 18,
    1999: 17,
    1998: 16,
    1997: 15,
    1996: 14,
    1995: 13,
    1994: 12,
    1993: 11,
    1992: 10,
    1991: 9,
    1990: 8,
    1985: 7,
    1980: 6,
    1975: 5,
    1970: 4,
    1965: 3,
    1960: 2,
    1955: 1,
    1950: 0,
}

ALLOWED_BOOLEAN_FILTERS: Dict[str, Dict[str, Any]] = {
    "new": {"cond": 1},
    "old": {"cond": 2},
    "private": {"f": "p"},
    "professional": {"f": "c"},
    "gasoline": {"fu": 1},
    "alcohol": {"fu": 2},
    "flex": {"fu": 3},
    "gas": {"fu": 4},
    "diesel": {"fu": 5},
    "manual": {"gb": 1},
    "automatic": {"gb": 2},
    "semi_automatic": {"gb": 3},
    "passenger": {"ctp": 1},
    "convertible": {"ctp": 2},
    "pick-up": {"ctp": 3},
    "vintage": {"ctp": 4},
    "suv": {"ctp": 5},
    "buggy": {"ctp": 6},
    "van": {"ctp": 7},
    "sedan": {"ctp": 8},
    "hatch": {"ctp": 9},
    "air_conditioner": {"cf": 1},
    "hydraulic_steering": {"cf": 2},
    "electric_lock": {"cf": 3},
    "electric_window": {"cf": 4},
    "air_bag": {"cf": 5},
    "alarm": {"cf": 6},
    "sound": {"cf": 7},
    "rear_sensor": {"cf": 8},
    "rear_camera": {"cf": 9},
    "armored": {"cf": 10},
}

ALLOWED_DYNAMIC_FILTERS: Dict[str, Dict[str, Any]] = {
    "min_mileage": {
        "filter_name": "ms",
        "has_parse_dict": True,
        "parse_dict": MILEAGE,
    },
    "max_mileage": {
        "filter_name": "me",
        "has_parse_dict": True,
        "parse_dict": MILEAGE,
    },
    "min_year": {
        "filter_name": "rs",
        "has_parse_dict": True,
        "parse_dict": YEAR,
    },
    "max_year": {
        "filter_name": "re",
        "has_parse_dict": True,
        "parse_dict": YEAR,
    },
    "min_price": {
        "filter_name": "ps",
        "has_parse_dict": False,
    },
    "max_price": {
        "filter_name": "pe",
        "has_parse_dict": False,
    },
}



import contextlib
from math import ceil
from typing import Any, Dict

from bs4 import BeautifulSoup

from olxbrasil.parsers.base import OlxBaseParser
from olxbrasil.utils import format_price


class ListParser(OlxBaseParser):
    def __init__(self, soup: BeautifulSoup):
        """
        Parser utilizado para listagem de items
        :param soup: Objeto SOAP para ser utilizado na busca de dados
        """
        super().__init__(soup)
        self.page_limit = self.__get_page_limit()
        self.current_page = self.__get_current_page()
        self.page_size = self.__get_page_size()

    def __get_page_limit(self) -> int:
        return min(
            ceil(self.__get_items_total() / self.__get_page_size()), 100
        )

    def __get_current_page(self) -> int:
        return self.initial_data["listingProps"]["pageIndex"]

    def __get_page_size(self) -> int:
        return self.initial_data["listingProps"]["pageSize"]

    def _get_ad_data(self):
        return self.initial_data["listingProps"]["adList"]

    def __get_items_total(self):
        return self.initial_data["listingProps"]["totalOfAds"]

    @property
    def items(self) -> Dict[str, Any]:
        ads = []
        for item in self.ad_data:
            with contextlib.suppress(KeyError):
                title = item.get("subject", "").strip()
                price = format_price(item.get("price"))
                url = item["url"]

                ads.append(
                    {
                        "title": title,
                        "price": float(price),
                        "url": url,
                        "is_professional": item["professionalAd"],
                    }
                )

        return {
            "ads": ads,
            "page": self.current_page,
            "total": self.page_size,
            "page_limit": self.page_limit,
        }



import abc
from typing import Dict, Iterable, Optional, Union

from olxbrasil.constants import LOCATIONS_URL
from olxbrasil.exceptions import FilterNotFoundError
from olxbrasil.utils import build_boolean_parameters, build_search_parameters


class Filter(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def get_filters(self, params: Optional[Dict] = None) -> Dict:
        pass  # pragma: nocover

    @abc.abstractmethod
    def get_endpoint(self) -> str:
        pass  # pragma: nocover


class ItemFilter(Filter):
    def __init__(
        self,
        *,
        manufacturer: Optional[str] = None,
        model: Optional[str] = None,
        boolean_filters: Optional[Iterable] = tuple(),
        search_filters: Optional[Dict] = None,
    ):
        """
        Filtro de items

        :param manufacturer: Fabricante do carro (Aplicável somente a veículos)
        :param model: Modelo do carro (Aplicável somente a veículos)
        :param boolean_filters: Filtros de ativo/inativo
        :param search_filters: Filtros de busca,
        como valor minimo ou valor maximo
        """
        self.__manufacturer = manufacturer
        self.__model = model
        self.__boolean_filters = boolean_filters
        self.__search_filters = search_filters

    def get_filters(self, params: Optional[Dict] = None) -> Dict:
        """
        Formata os filtros para o formato esperado pela OLX

        :param params: Parametros para serem formatados
        :return: Dicionário com os parametros
        para ser utilizado na formação da URL
        """
        item_filter = params or {}
        if self.__boolean_filters:
            item_filter.update(
                build_boolean_parameters(*self.__boolean_filters)
            )
        if self.__search_filters:
            item_filter.update(
                build_search_parameters(**self.__search_filters)
            )

        return item_filter

    def get_endpoint(self) -> str:
        """
        Em caso de veículo retorna o endpoint
        para ser utilizado na formação da URL

        :return: Endpoint para ser utilizado na formação da URL
        """
        endpoint = ""

        if self.__manufacturer:
            endpoint += f"/{self.__manufacturer.lower()}"
            if self.__model:
                endpoint += f"/{self.__model.lower()}"

        return endpoint


class LocationFilter(Filter):
    def __init__(self, state: str, ddd: Optional[Union[int, float]] = None):
        """
        Filtro de localização

        :param state: Estado onde vai ser feito a busca
        :param ddd: DDD da região da busca,
        tem que ser um DDD pertencente ao estado escolhido
        """
        self.state = state.upper()
        self.__ddd = ddd
        self.__validate()

    def __validate(self) -> bool:
        """Valida se o estado existe e se o DDD pertence ao estado escolhido"""
        if self.state not in LOCATIONS_URL:
            raise FilterNotFoundError(f"State {self.state} not found")
        if self.__ddd and self.__ddd not in LOCATIONS_URL[self.state]:
            raise FilterNotFoundError(
                f"DDD {self.__ddd} was not found in state {self.state}"
            )

        return True

    def get_filters(
        self, params: Optional[Dict] = None
    ) -> Dict:  # pragma: nocover
        pass

    def get_endpoint(self) -> str:
        """Retorna o endpoint da localização
        para ser utilizado na formatação da URL"""
        if self.__ddd:
            return LOCATIONS_URL[self.state][self.__ddd]
        return ""



from typing import Any, Dict, Optional

from bs4 import BeautifulSoup
from fake_useragent import UserAgent
from httpx import AsyncClient, Client, ConnectError, HTTPStatusError

from olxbrasil.constants import CATEGORIES
from olxbrasil.exceptions import OlxRequestError
from olxbrasil.filters import Filter, LocationFilter
from olxbrasil.parsers import ItemParser, ListParser

user_agent = UserAgent()


class Olx:
    def __init__(
        self,
        *,
        category: str,
        subcategory: Optional[str] = None,
        location: Optional[LocationFilter] = None,
        filters: Optional[Filter] = None,
    ):
        """
        Parser síncrono para OLX Brasil
        :param category: Categoria utilizada na busca da OLX
        :param subcategory: Subcategoria utilizada na
        busca da OLX, deve pertencer a categoria
        :param location: Objeto de filtro de localidade
        :param filters: Objeto de filtro de parametros
        """
        if not location:
            self.location = None
            self.subdomain = "www"
        else:
            self.location = location
            self.subdomain = self.location.state.lower()
        self.user_agent = user_agent
        self.category = None
        self.subcategory = None
        self.client = Client(
            base_url=f"https://{self.subdomain}.olx.com.br",
            headers={"User-Agent": self.user_agent.random},
        )
        self.filters = filters

        valid_category = category in CATEGORIES.keys()

        if valid_category:
            valid_subcategory = (
                subcategory in CATEGORIES[category]["subcategories"].keys()
            )
            self.category = CATEGORIES[category]["category"]

            if subcategory and valid_subcategory:
                sub = CATEGORIES[category]["subcategories"][subcategory]
                self.subcategory = sub

            if subcategory and not valid_subcategory:
                raise ValueError(
                    f"{subcategory} is not a valid subcategory, "
                    "please provide a valid subcategory: "
                    f"{' '.join(CATEGORIES[category]['subcategories'].keys())}"
                )
        else:
            raise ValueError(
                f"{category} is not a valid category, "
                "please provide a valid category: "
                f"{' '.join(CATEGORIES.keys())}"
            )

    def build_url(self) -> str:
        url = ""
        if self.location:
            url += self.location.get_endpoint()

        url += f"/{self.category}"

        if self.subcategory:
            url += f"/{self.subcategory}"

        if self.filters:
            url += self.filters.get_endpoint()

        return url

    def fetch_all(self, page=0) -> Dict[str, Any]:
        parameters = {"o": min(page, 100)}
        url = self.build_url()

        if self.filters:
            parameters = self.filters.get_filters(parameters)

        try:
            response = self.client.get(url, params=parameters)

            response.raise_for_status()
        except HTTPStatusError as err:
            raise OlxRequestError(
                "Was not possible to reach OLX server"
            ) from err

        soup = BeautifulSoup(response.text, "html.parser")

        parser = ListParser(soup)

        return parser.items

    def fetch_item(self, url: str) -> ItemParser:
        try:
            response = self.client.get(url)

            response.raise_for_status()
        except (HTTPStatusError, ConnectError) as err:
            raise OlxRequestError(
                "Was not possible to reach OLX server"
            ) from err

        soup = BeautifulSoup(response.text, "html.parser")
        parser = ItemParser(soup)
        return parser


class AsyncOlx(Olx):
    def __init__(
        self,
        *,
        category: str,
        subcategory: Optional[str] = None,
        location: Optional[LocationFilter] = None,
        filters: Optional[Filter] = None,
    ):
        super().__init__(
            category=category,
            subcategory=subcategory,
            location=location,
            filters=filters,
        )
        self.__base_url: str = f"https://{self.subdomain}.olx.com.br"
        self.__headers: Dict[str, str] = {"User-Agent": self.user_agent.random}

    async def fetch_all(  # type: ignore
        self, page: Optional[int] = 0
    ) -> Dict[str, Any]:
        parameters = {"o": min(page, 100)}  # type: ignore
        url = self.build_url()

        if self.filters:
            parameters = self.filters.get_filters(parameters)

        try:
            async with AsyncClient(
                base_url=self.__base_url,
                headers=self.__headers,
            ) as client:
                response = await client.get(url, params=parameters)

            response.raise_for_status()
        except (HTTPStatusError, ConnectError) as err:
            raise OlxRequestError(
                "Was not possible to reach OLX server"
            ) from err

        soup = BeautifulSoup(response.text, "html.parser")

        parser = ListParser(soup)

        return parser.items

    async def fetch_item(self, url: str) -> ItemParser:  # type: ignore
        try:
            async with AsyncClient(
                base_url=self.__base_url,
                headers=self.__headers,
            ) as client:
                response = await client.get(url)

                response.raise_for_status()
        except (HTTPStatusError, ConnectError) as err:
            raise OlxRequestError(
                "Was not possible to reach OLX server"
            ) from err

        soup = BeautifulSoup(response.text, "html.parser")
        parser = ItemParser(soup)
        return parser



"""Biblioteca para scrapping da OLX."""
from olxbrasil.exceptions import FilterNotFoundError, OlxRequestError
from olxbrasil.filters import ItemFilter, LocationFilter
from olxbrasil.service import AsyncOlx, Olx

__all__ = (
    "Olx",
    "AsyncOlx",
    "ItemFilter",
    "LocationFilter",
    "OlxRequestError",
    "FilterNotFoundError",
)



from olxbrasil.parsers.base import OlxBaseParser
from olxbrasil.utils import format_price


class ItemParser(OlxBaseParser):
    @property
    def title(self) -> str:
        return self.ad_data["subject"].strip()

    @property
    def price(self) -> float:
        return format_price(self.ad_data["price"])

    @property
    def seller(self) -> str:
        return self.ad_data["user"]["name"].strip()

    @property
    def description(self) -> str:
        return self.ad_data["description"].replace("<br>", "\n").strip()

    @property
    def phone(self) -> str:
        return self.ad_data["phone"]["phone"].strip()

    @property
    def location(self) -> dict:
        return self.ad_data["location"]

    @property
    def properties(self) -> dict:
        car_properties = {}
        for item in self.ad_data["properties"]:
            car_properties[item["name"]] = item["value"]
        return car_properties



import json
from typing import Generator

from bs4 import BeautifulSoup


class OlxBaseParser:
    def __init__(self, soup: BeautifulSoup):
        """
        Parser base para todos os parsers derivados

        :param soup: Objeto SOAP para ser utilizado na busca de dados
        """
        self.soup = soup
        self.initial_data = self.__get_initial_data()
        self.ad_data = self._get_ad_data()

    def __iter__(self) -> Generator:
        """
        Implementa iterador para que seja possível aplica dict(Parser)

        :return: Tupla com nome do campo e valor
        """
        for item in dir(self):
            attr = getattr(self, item)
            if (
                not item.startswith("_")
                and item
                not in (
                    "initial_data",
                    "ad_data",
                    "soup",
                )
                and not callable(attr)
            ):
                yield item, getattr(self, item)

    def __getitem__(self, item):
        """
        Implementa getitem para que seja possível
        buscar atributo com Parser['atributo']

        :param item: Chave a ser buscada
        :return: Valor atrelado a chave
        """
        try:
            return getattr(self, item)
        except AttributeError:
            return None

    def _get_ad_data(self) -> dict:
        return self.initial_data.get("ad", {})

    def __get_initial_data(self) -> dict:
        tag = "script"
        options = {"id": "initial-data"}
        key = "data-json"
        return json.loads(self.soup.find(tag, options)[key])



from typing import Any, Dict

from olxbrasil.constants import (
    ALLOWED_BOOLEAN_FILTERS,
    ALLOWED_DYNAMIC_FILTERS,
)


def format_price(price: str = "0") -> float:
    try:
        price_value = (
            price.replace("R$ ", "").replace(".", "").replace(",", ".")
        )
        return float(price_value)
    except (ValueError, AttributeError):
        return 0


def build_boolean_parameters(*olx_filters: str) -> dict:
    params: Dict[Any, Any] = {}
    for olx_filter in olx_filters:
        if olx_filter not in ALLOWED_BOOLEAN_FILTERS:
            continue

        for key, value in ALLOWED_BOOLEAN_FILTERS[olx_filter].items():
            append_parameter(params, key, value)

    return params


def build_search_parameters(**parameters) -> dict:
    params: Dict[Any, Any] = {}
    for search_filter, value in parameters.items():
        if search_filter not in ALLOWED_DYNAMIC_FILTERS:
            continue

        chosen_value = value
        chosen_filter = ALLOWED_DYNAMIC_FILTERS[search_filter]
        if chosen_filter.get("has_parse_dict"):
            try:
                chosen_value = chosen_filter["parse_dict"][value]
            except KeyError:
                chosen_value = list(chosen_filter["parse_dict"].values())[0]

        append_parameter(params, chosen_filter["filter_name"], chosen_value)

    return params


def append_parameter(parameters: dict, parameter: str, value: Any) -> None:
    if parameter in parameters and not isinstance(parameters[parameter], list):
        parameters[parameter] = sorted([parameters[parameter], value])
    elif parameter not in parameters:
        parameters[parameter] = value
    else:
        parameters[parameter].append(value)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['olxbrasil', 'olxbrasil.parsers']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.9.3,<5.0.0',
 'fake-useragent>=0.1.11,<0.2.0',
 'httpx>=0.19.0,<0.20.0']

setup_kwargs = {
    'name': 'olxbrasil',
    'version': '0.2.5',
    'description': 'Biblioteca para scrapping da Olx Brasil (olx.com.br)',
    'long_description': '# OLX Brasil Scrapping\n\n[![MIT License](https://img.shields.io/badge/license-MIT-007EC7.svg?style=flat-square)](/LICENSE)\n[![PyPI](https://img.shields.io/pypi/v/olxbrasil.svg)](https://pypi.python.org/pypi/olxbrasil)\n[![PyPI](https://img.shields.io/pypi/pyversions/olxbrasil.svg)]()\n![PyPI - Downloads](https://img.shields.io/pypi/dm/olxbrasil.svg?label=pip%20installs&logo=python)\n[![codecov](https://codecov.io/gh/mdslino/olxbrasil/branch/main/graph/badge.svg)](https://codecov.io/gh/mdslino/olxbrasil)\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/54968a88b27647f1b47154f942905a5d)](https://www.codacy.com/gh/Mdslino/olxbrasil/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=Mdslino/olxbrasil&amp;utm_campaign=Badge_Grade)\n![GitHub issues](https://img.shields.io/github/issues/mdslino/olxbrasil.svg)\n![GitHub stars](https://img.shields.io/github/stars/mdslino/olxbrasil.svg)\n![GitHub last commit](https://img.shields.io/github/last-commit/mdslino/olxbrasil.svg)\n[![Code Style Black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black/)\n',
    'author': 'Marcelo Lino',
    'author_email': 'mdslino@gmail.com',
    'maintainer': 'Marcelo Lino',
    'maintainer_email': 'mdslino@gmail.com',
    'url': 'https://github.com/Mdslino/olxbrasil',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
